@@ -23,9 +23,9 @@
  * One event can trigger multiple feedbacks at once (e.g. audio and
  * haptic feedback). This is determined by the feedback theme in
  * use (which is not under the appliction's control) and the active
- * feedback profile (see [func@Lfb.set_feedback_profile]().
+ * feedback profile (see [func@Lfb.set_feedback_profile].
  *
- * After initializing the library via [func@Lfb.init]() feedback can be
+ * After initializing the library via [func@Lfb.init] feedback can be
  * triggered like:
  *
  * ```c
@@ -38,7 +38,7 @@
  *
  * When all feedback for this event has ended the [signal@LfbEvent::feedback-ended]
  * signal is emitted. If you want to end the feedback ahead of time use
- * [method@LfbEvent.end_feedback]():
+ * [method@LfbEvent.end_feedback]:
  *
  * ```c
  *   if (!lfb_event_end_feedback (event, &err))
@@ -46,7 +46,7 @@
  * ```
  *
  * Since these methods involve DBus calls there are asynchronous variants
- * available, e.g. [method@LfbEvent.trigger_feedback_async]():
+ * available, e.g. [method@LfbEvent.trigger_feedback_async]:
  *
  * ```c
  *   static void
@@ -79,6 +79,7 @@ enum {
   PROP_STATE,
   PROP_END_REASON,
   PROP_FEEDBACK_PROFILE,
+  PROP_APP_ID,
   PROP_LAST_PROP,
 };
 static GParamSpec *props[PROP_LAST_PROP];
@@ -95,6 +96,7 @@ typedef struct _LfbEvent {
   char          *event;
   gint           timeout;
   gchar         *profile;
+  char          *app_id;
 
   guint          id;
   LfbEventState  state;
@@ -221,6 +223,9 @@ lfb_event_set_property (GObject      *object,
   case PROP_FEEDBACK_PROFILE:
     lfb_event_set_feedback_profile (self, g_value_get_string (value));
     break;
+  case PROP_APP_ID:
+    lfb_event_set_app_id (self, g_value_get_string (value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     break;
@@ -244,7 +249,10 @@ lfb_event_get_property (GObject    *object,
     g_value_set_int (value, self->timeout);
     break;
   case PROP_FEEDBACK_PROFILE:
-    g_value_set_string (value, self->profile);
+    g_value_set_string (value, lfb_event_get_feedback_profile (self));
+    break;
+  case PROP_APP_ID:
+    g_value_set_string (value, lfb_event_get_app_id (self));
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -262,6 +270,7 @@ lfb_event_finalize (GObject *object)
 
   g_clear_pointer (&self->event, g_free);
   g_clear_pointer (&self->profile, g_free);
+  g_clear_pointer (&self->app_id, g_free);
 
   G_OBJECT_CLASS (lfb_event_parent_class)->finalize (object);
 }
@@ -294,7 +303,7 @@ lfb_event_class_init (LfbEventClass *klass)
    *
    * How long feedback should be provided in seconds. The special value
    * %-1 uses the natural length of each feedback while %0 plays each feedback
-   * in a loop until ended explicitly via e.g. [method@LfbEvent.end_feedback]().
+   * in a loop until ended explicitly via e.g. [method@LfbEvent.end_feedback].
    */
   props[PROP_TIMEOUT] =
     g_param_spec_int (
@@ -326,7 +335,7 @@ lfb_event_class_init (LfbEventClass *klass)
    * LfbEvent:feedback-profile:
    *
    * The name of the feedback profile to use for this event. See
-   * [method@LfbEvent.set_feedback_profile]() for details.
+   * [method@LfbEvent.set_feedback_profile] for details.
    */
   props[PROP_FEEDBACK_PROFILE] =
     g_param_spec_string (
@@ -336,6 +345,20 @@ lfb_event_class_init (LfbEventClass *klass)
       NULL,
       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * LfbEvent:app-id:
+   *
+   * The application id to use for the event.
+   * [method@LfbEvent.set_feedback_profile] for details.
+   */
+  props[PROP_APP_ID] =
+    g_param_spec_string (
+      "app-id",
+      "Application Id",
+      "The Application id to use for this event",
+      NULL,
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, PROP_LAST_PROP, props);
 
   /**
@@ -411,6 +434,7 @@ lfb_event_trigger_feedback (LfbEvent *self, GError **error)
 {
   LfbGdbusFeedback *proxy;
   gboolean success;
+  const char *app_id;
 
   g_return_val_if_fail (LFB_IS_EVENT (self), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
@@ -429,8 +453,9 @@ lfb_event_trigger_feedback (LfbEvent *self, GError **error)
                                                  G_CONNECT_SWAPPED);
    }
 
+   app_id = self->app_id ?: lfb_get_app_id ();
    success =  lfb_gdbus_feedback_call_trigger_feedback_sync (proxy,
-                                                             lfb_get_app_id (),
+                                                             app_id,
                                                              self->event,
                                                              build_hints (self),
                                                              self->timeout,
@@ -452,7 +477,7 @@ lfb_event_trigger_feedback (LfbEvent *self, GError **error)
  *
  * Tells the feedback server to provide proper feedback for the give
  * event to the user. This is the sync version of
- * [method@LfbEvent.trigger_feedback]().
+ * [method@LfbEvent.trigger_feedback].
  */
 void
 lfb_event_trigger_feedback_async (LfbEvent            *self,
@@ -462,6 +487,7 @@ lfb_event_trigger_feedback_async (LfbEvent            *self,
 {
   LfbAsyncData *data;
   LfbGdbusFeedback *proxy;
+  const char *app_id;
 
   g_return_if_fail (LFB_IS_EVENT (self));
   if (!lfb_is_initted ())
@@ -481,8 +507,10 @@ lfb_event_trigger_feedback_async (LfbEvent            *self,
   data = g_new0 (LfbAsyncData, 1);
   data->task = g_task_new (self, cancellable, callback, user_data);
   data->event = g_object_ref (self);
+
+  app_id = self->app_id ?: lfb_get_app_id ();
   lfb_gdbus_feedback_call_trigger_feedback (proxy,
-                                            lfb_get_app_id (),
+                                            app_id,
                                             self->event,
                                             build_hints (self),
                                             self->timeout,
@@ -497,7 +525,7 @@ lfb_event_trigger_feedback_async (LfbEvent            *self,
  * @res: Result object passed to the callback of [method@LfbEvent.trigger_feedback_async]
  * @error: Return location for error
  *
- * Finish an async operation started by [method@LfbEvent.trigger_feedback_async](). You
+ * Finish an async operation started by [method@LfbEvent.trigger_feedback_async]. You
  * must call this function in the callback to free memory and receive any
  * errors which occurred.
  *
@@ -543,7 +571,7 @@ lfb_event_end_feedback (LfbEvent *self, GError **error)
 /**
  * lfb_event_end_feedback_finish:
  * @self: the event
- * @res: Result object passed to the callback of [method@LfbEvent.end_feedback_async]()
+ * @res: Result object passed to the callback of [method@LfbEvent.end_feedback_async]
  * @error: Return location for error
  *
  * Finish an async operation started by lfb_event_end_feedback_async. You
@@ -609,10 +637,10 @@ lfb_event_end_feedback_async (LfbEvent            *self,
  * Tells the feedback server to end feedack after #timeout seconds.
  * The value -1 indicates to not set a timeout and let feedbacks stop
  * on their own while 0 indicates to loop all feedbacks endlessly.
- * They must be stopped via [method@LfbEvent.end_feedback]() in this case.
+ * They must be stopped via [method@LfbEvent.end_feedback] in this case.
  *
  * It is an error to change the timeout after the feedback has been triggered
- * via [method@LfbEvent.trigger_feedback]().
+ * via [method@LfbEvent.trigger_feedback].
  */
 void
 lfb_event_set_timeout (LfbEvent *self, gint timeout)
@@ -716,13 +744,60 @@ lfb_event_set_feedback_profile (LfbEvent *self, const gchar *profile)
  * lfb_event_get_feedback_profile:
  * @self: The event
  *
- * Returns:(transfer full): The set feedback profile to use for this
- * event or %NULL.
+ * Gets the set feedback profile. If no profile was set it returns
+ * %NULL. The event uses the system wide profile in this case.
+ *
+ * Returns: The set feedback profile to use for this event or %NULL.
  */
-char *
+const char *
 lfb_event_get_feedback_profile (LfbEvent *self)
 {
   g_return_val_if_fail (LFB_IS_EVENT (self), NULL);
 
-  return g_strdup (self->profile);
+  return self->profile;
+}
+
+/**
+ * lfb_event_set_app_id:
+ * @self: The event
+ * @app_id: The application id to use
+ *
+ * Tells the feedback server to use the given application id for
+ * this event when it is submitted. The server might ignore this
+ * request. This can be used by notification daemons to honor per
+ * application settings automatically.
+ *
+ * The functions is usually not used by applications.
+ *
+ * A value of %NULL (the default) lets the server pick the profile.
+ */
+void
+lfb_event_set_app_id (LfbEvent *self, const gchar *app_id)
+{
+  g_return_if_fail (LFB_IS_EVENT (self));
+
+  if (!g_strcmp0 (self->app_id, app_id))
+    return;
+
+  g_free (self->app_id);
+  self->app_id = g_strdup (app_id);
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_APP_ID]);
+}
+
+/**
+ * lfb_event_get_app_id:
+ * @self: The event
+ *
+ * Returns the app-id for this event. If no app-id has been explicitly
+ * set, %NULL is returned. The event uses the app-id returned by
+ * [func@Lfb.get_app_id] in this case.
+ *
+ * Returns:(transfer none): The set app-id for this event or %NULL.
+ */
+const char *
+lfb_event_get_app_id (LfbEvent *self)
+{
+  g_return_val_if_fail (LFB_IS_EVENT (self), NULL);
+
+  return self->app_id;
 }
